@@ -19,23 +19,13 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-/**
- * SECTION:completionitem
- * @title: GtkSourceCompletionItem
- * @short_description: Simple implementation of GtkSourceCompletionProposal
- *
- * The #GtkSourceCompletionItem class is a simple implementation of the
- * #GtkSourceCompletionProposal interface.
- */
-
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include "gtksourcecompletionitem.h"
 #include "gtksourcecompletionproposal.h"
+#include "gtksourcecompletionutils.h"
 #include "gtksourceview-i18n.h"
 
+#define GTK_SOURCE_COMPLETION_ITEM_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE((object), GTK_SOURCE_TYPE_COMPLETION_ITEM, GtkSourceCompletionItemPrivate))
+
 struct _GtkSourceCompletionItemPrivate
 {
 	gchar *label;
@@ -43,10 +33,9 @@ struct _GtkSourceCompletionItemPrivate
 	gchar *text;
 	gchar *info;
 	GdkPixbuf *icon;
-	gchar *icon_name;
-	GIcon *gicon;
 };
 
+/* Properties */
 enum
 {
 	PROP_0,
@@ -54,8 +43,6 @@ enum
 	PROP_MARKUP,
 	PROP_TEXT,
 	PROP_ICON,
-	PROP_ICON_NAME,
-	PROP_GICON,
 	PROP_INFO
 };
 
@@ -64,7 +51,6 @@ static void gtk_source_completion_proposal_iface_init (gpointer g_iface, gpointe
 G_DEFINE_TYPE_WITH_CODE (GtkSourceCompletionItem,
 			 gtk_source_completion_item,
 			 G_TYPE_OBJECT,
-			 G_ADD_PRIVATE (GtkSourceCompletionItem)
 			 G_IMPLEMENT_INTERFACE (GTK_SOURCE_TYPE_COMPLETION_PROPOSAL,
 			 			gtk_source_completion_proposal_iface_init))
 
@@ -92,18 +78,6 @@ gtk_source_completion_proposal_get_icon_impl (GtkSourceCompletionProposal *self)
 	return GTK_SOURCE_COMPLETION_ITEM (self)->priv->icon;
 }
 
-static const gchar *
-gtk_source_completion_proposal_get_icon_name_impl (GtkSourceCompletionProposal *self)
-{
-	return GTK_SOURCE_COMPLETION_ITEM (self)->priv->icon_name;
-}
-
-static GIcon *
-gtk_source_completion_proposal_get_gicon_impl (GtkSourceCompletionProposal *self)
-{
-	return GTK_SOURCE_COMPLETION_ITEM (self)->priv->gicon;
-}
-
 static gchar *
 gtk_source_completion_proposal_get_info_impl (GtkSourceCompletionProposal *self)
 {
@@ -121,8 +95,6 @@ gtk_source_completion_proposal_iface_init (gpointer g_iface,
 	iface->get_markup = gtk_source_completion_proposal_get_markup_impl;
 	iface->get_text = gtk_source_completion_proposal_get_text_impl;
 	iface->get_icon = gtk_source_completion_proposal_get_icon_impl;
-	iface->get_icon_name = gtk_source_completion_proposal_get_icon_name_impl;
-	iface->get_gicon = gtk_source_completion_proposal_get_gicon_impl;
 	iface->get_info = gtk_source_completion_proposal_get_info_impl;
 }
 
@@ -142,13 +114,6 @@ gtk_source_completion_item_finalize (GObject *object)
 		g_object_unref (self->priv->icon);
 	}
 
-	g_free (self->priv->icon_name);
-
-	if (self->priv->gicon != NULL)
-	{
-		g_object_unref (self->priv->gicon);
-	}
-
 	G_OBJECT_CLASS (gtk_source_completion_item_parent_class)->finalize (object);
 }
 
@@ -181,12 +146,6 @@ gtk_source_completion_item_get_property (GObject    *object,
 		case PROP_ICON:
 			g_value_set_object (value, self->priv->icon);
 			break;
-		case PROP_ICON_NAME:
-			g_value_set_string (value, self->priv->icon_name);
-			break;
-		case PROP_GICON:
-			g_value_set_object (value, self->priv->gicon);
-			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 			break;
@@ -216,11 +175,13 @@ gtk_source_completion_item_set_property (GObject      *object,
 		case PROP_LABEL:
 			g_free (self->priv->label);
 			self->priv->label = g_value_dup_string (value);
+
 			emit_changed (self);
 			break;
 		case PROP_MARKUP:
 			g_free (self->priv->markup);
 			self->priv->markup = g_value_dup_string (value);
+
 			emit_changed (self);
 			break;
 		case PROP_TEXT:
@@ -230,6 +191,7 @@ gtk_source_completion_item_set_property (GObject      *object,
 		case PROP_INFO:
 			g_free (self->priv->info);
 			self->priv->info = g_value_dup_string (value);
+
 			emit_changed (self);
 			break;
 		case PROP_ICON:
@@ -241,20 +203,6 @@ gtk_source_completion_item_set_property (GObject      *object,
 			self->priv->icon = GDK_PIXBUF (g_value_dup_object (value));
 			emit_changed (self);
 			break;
-		case PROP_ICON_NAME:
-			g_free (self->priv->icon_name);
-			self->priv->icon_name = g_value_dup_string (value);
-			emit_changed (self);
-			break;
-		case PROP_GICON:
-			if (self->priv->gicon != NULL)
-			{
-				g_object_unref (self->priv->gicon);
-			}
-
-			self->priv->gicon = G_ICON (g_value_dup_object (value));
-			emit_changed (self);
-			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 			break;
@@ -278,11 +226,10 @@ gtk_source_completion_item_class_init (GtkSourceCompletionItemClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_LABEL,
 					 g_param_spec_string ("label",
-							      "Label",
-							      "Label to be shown for this item",
+							      _("Label"),
+							      _("Label to be shown for this item"),
 							      NULL,
-							      G_PARAM_READWRITE |
-							      G_PARAM_STATIC_STRINGS));
+							      G_PARAM_READWRITE));
 
 	/**
 	 * GtkSourceCompletionItem:markup:
@@ -292,11 +239,10 @@ gtk_source_completion_item_class_init (GtkSourceCompletionItemClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_MARKUP,
 					 g_param_spec_string ("markup",
-							      "Markup",
-							      "Markup to be shown for this item",
+							      _("Markup"),
+							      _("Markup to be shown for this item"),
 							      NULL,
-							      G_PARAM_READWRITE |
-							      G_PARAM_STATIC_STRINGS));
+							      G_PARAM_READWRITE));
 
 	/**
 	 * GtkSourceCompletionItem:text:
@@ -306,57 +252,24 @@ gtk_source_completion_item_class_init (GtkSourceCompletionItemClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_TEXT,
 					 g_param_spec_string ("text",
-							      "Text",
-							      "Item text",
+							      _("Text"),
+							      _("Item text"),
 							      NULL,
-							      G_PARAM_READWRITE |
-							      G_PARAM_STATIC_STRINGS));
+							      G_PARAM_READWRITE));
 
 	/**
 	 * GtkSourceCompletionItem:icon:
 	 *
-	 * The #GdkPixbuf for the icon to be shown for this proposal.
+	 * Icon to be shown for this proposal.
 	 */
 	g_object_class_install_property (object_class,
 					 PROP_ICON,
 					 g_param_spec_object ("icon",
-							      "Icon",
-							      "Pixbuf of the icon to be shown for this item",
+							      _("Icon"),
+							      _("Icon to be shown for this item"),
 							      GDK_TYPE_PIXBUF,
-							      G_PARAM_READWRITE |
-							      G_PARAM_STATIC_STRINGS));
+							      G_PARAM_READWRITE));
 
-	/**
-	 * GtkSourceCompletionItem:icon-name:
-	 *
-	 * The icon name for the icon to be shown for this proposal.
-	 *
-	 * Since: 3.18
-	 */
-	g_object_class_install_property (object_class,
-					 PROP_ICON_NAME,
-					 g_param_spec_string ("icon-name",
-							      "Icon Name",
-							      "Icon name of the icon to be shown for this item",
-							      NULL,
-							      G_PARAM_READWRITE |
-							      G_PARAM_STATIC_STRINGS));
-
-	/**
-	 * GtkSourceCompletionItem:gicon:
-	 *
-	 * The #GIcon for the icon to be shown for this proposal.
-	 *
-	 * Since: 3.18
-	 */
-	g_object_class_install_property (object_class,
-					 PROP_GICON,
-					 g_param_spec_object ("gicon",
-							      "GIcon",
-							      "GIcon of the icon to be shown for this item",
-							      G_TYPE_ICON,
-							      G_PARAM_READWRITE |
-							      G_PARAM_STATIC_STRINGS));
 	/**
 	 * GtkSourceCompletionItem:info:
 	 *
@@ -365,25 +278,26 @@ gtk_source_completion_item_class_init (GtkSourceCompletionItemClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_INFO,
 					 g_param_spec_string ("info",
-							      "Info",
-							      "Info to be shown for this item",
+							      _("Info"),
+							      _("Info to be shown for this item"),
 							      NULL,
-							      G_PARAM_READWRITE |
-							      G_PARAM_STATIC_STRINGS));
+							      G_PARAM_READWRITE));
+
+	g_type_class_add_private (object_class, sizeof (GtkSourceCompletionItemPrivate));
 }
 
 static void
 gtk_source_completion_item_init (GtkSourceCompletionItem *self)
 {
-	self->priv = gtk_source_completion_item_get_instance_private (self);
+	self->priv = GTK_SOURCE_COMPLETION_ITEM_GET_PRIVATE (self);
 }
 
 /**
  * gtk_source_completion_item_new:
  * @label: The item label.
  * @text: The item text.
- * @icon: (nullable): The item icon.
- * @info: (nullable): The item extra information.
+ * @icon: (allow-none): The item icon.
+ * @info: (allow-none): The item extra information.
  *
  * Create a new #GtkSourceCompletionItem with label @label, icon @icon and
  * extra information @info. Both @icon and @info can be %NULL in which case
@@ -409,8 +323,8 @@ gtk_source_completion_item_new (const gchar *label,
  * gtk_source_completion_item_new_with_markup:
  * @markup: The item markup label.
  * @text: The item text.
- * @icon: (nullable): The item icon.
- * @info: (nullable): The item extra information.
+ * @icon: (allow-none): The item icon.
+ * @info: (allow-none): The item extra information.
  *
  * Create a new #GtkSourceCompletionItem with markup label @markup, icon
  * @icon and extra information @info. Both @icon and @info can be %NULL in
@@ -432,20 +346,17 @@ gtk_source_completion_item_new_with_markup (const gchar *markup,
 			     NULL);
 }
 
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
-
 /**
  * gtk_source_completion_item_new_from_stock:
- * @label: (nullable): The item label.
+ * @label: (allow-none): The item label.
  * @text: The item text.
  * @stock: The stock icon.
- * @info: (nullable): The item extra information.
+ * @info: (allow-none): The item extra information.
  *
  * Creates a new #GtkSourceCompletionItem from a stock item. If @label is %NULL,
  * the stock label will be used.
  *
  * Returns: a new #GtkSourceCompletionItem.
- * Deprecated: 3.10: Use gtk_source_completion_item_new() instead.
  */
 GtkSourceCompletionItem *
 gtk_source_completion_item_new_from_stock (const gchar *label,
@@ -491,5 +402,3 @@ gtk_source_completion_item_new_from_stock (const gchar *label,
 
 	return item;
 }
-
-G_GNUC_END_IGNORE_DEPRECATIONS;
