@@ -23,9 +23,8 @@
 #define __GTK_SOURCE_COMPLETION_MODEL_H__
 
 #include <gtk/gtk.h>
-#include <gtksourceview/gtksourcetypes.h>
-#include <gtksourceview/gtksourcecompletionprovider.h>
-#include <gtksourceview/gtksourcecompletionproposal.h>
+#include "gtksourcetypes.h"
+#include "gtksourcetypes-private.h"
 
 G_BEGIN_DECLS
 
@@ -64,46 +63,67 @@ enum
 	GTK_SOURCE_COMPLETION_MODEL_N_COLUMNS
 };
 
+G_GNUC_INTERNAL
 GType gtk_source_completion_model_get_type (void) G_GNUC_CONST;
 
+G_GNUC_INTERNAL
 GtkSourceCompletionModel *
 		gtk_source_completion_model_new 	(void);
 
+G_GNUC_INTERNAL
 void		gtk_source_completion_model_begin	(GtkSourceCompletionModel           *model,
                                                          GList                              *providers);
+
+G_GNUC_INTERNAL
 void		gtk_source_completion_model_append 	(GtkSourceCompletionModel           *model,
 							 GtkSourceCompletionProvider        *provider,
 							 GList                              *proposals);
+
+G_GNUC_INTERNAL
 void		gtk_source_completion_model_end		(GtkSourceCompletionModel           *model,
 							 GtkSourceCompletionProvider        *provider);
+
+G_GNUC_INTERNAL
 void		gtk_source_completion_model_cancel	(GtkSourceCompletionModel           *model);
 
+G_GNUC_INTERNAL
 gboolean	gtk_source_completion_model_is_empty 	(GtkSourceCompletionModel           *model,
                                                          gboolean                            invisible);
 
+G_GNUC_INTERNAL
 void            gtk_source_completion_model_set_visible_providers (GtkSourceCompletionModel *model,
                                                                    GList                    *providers);
+
+G_GNUC_INTERNAL
 GList          *gtk_source_completion_model_get_visible_providers (GtkSourceCompletionModel *model);
 
-
+G_GNUC_INTERNAL
 GList          *gtk_source_completion_model_get_providers (GtkSourceCompletionModel         *model);
+
+G_GNUC_INTERNAL
 guint		gtk_source_completion_model_n_proposals (GtkSourceCompletionModel           *model,
                                                          GtkSourceCompletionProvider        *provider);
 
+G_GNUC_INTERNAL
 void 		gtk_source_completion_model_clear 	(GtkSourceCompletionModel           *model);
 
+G_GNUC_INTERNAL
 void 		gtk_source_completion_model_set_show_headers (GtkSourceCompletionModel      *model,
 							      gboolean                       show_headers);
 
+G_GNUC_INTERNAL
 gboolean	gtk_source_completion_model_iter_is_header (GtkSourceCompletionModel        *model,
                                                             GtkTreeIter                     *iter);
 
+G_GNUC_INTERNAL
 gboolean 	gtk_source_completion_model_iter_previous (GtkSourceCompletionModel         *model,
 							   GtkTreeIter                      *iter);
 
+G_GNUC_INTERNAL
 gboolean 	gtk_source_completion_model_iter_last 	(GtkSourceCompletionModel           *model,
 							 GtkTreeIter                        *iter);
 
+G_GNUC_INTERNAL
 gboolean	gtk_source_completion_model_iter_equal	(GtkSourceCompletionModel           *model,
 							 GtkTreeIter                        *iter1,
 							 GtkTreeIter                        *iter2);
